@@ -4,8 +4,8 @@ use crate::history::History;
 use crate::fixed_length_grapheme_string::FixedLengthGraphemeString;
 use crate::history::Command;
 use crate::history_cleaner;
-use crate::settings::Settings;
 use crate::settings::{InterfaceView, KeyScheme};
+use crate::settings::{ResultSort, Settings};
 use chrono::{Duration, TimeZone, Utc};
 use humantime::format_duration;
 use std::io::{stdin, stdout, Write};
@@ -27,6 +27,7 @@ pub struct Interface<'a> {
     delete_requests: Vec<String>,
     menu_mode: MenuMode,
     in_vim_insert_mode: bool,
+    result_sort: ResultSort,
 }
 
 pub struct SelectionResult {
@@ -50,20 +51,33 @@ pub enum MenuMode {
 }
 
 impl MenuMode {
-    fn text(&self, interface: &Interface) -> &str {
+    fn text(&self, interface: &Interface) -> String {
+        let mut menu_text = String::from("McFly");
         match *self {
             MenuMode::Normal => match interface.settings.key_scheme {
-                KeyScheme::Emacs => "McFly | ESC - Exit | ⏎ - Run | TAB - Edit | F2 - Delete",
+                KeyScheme::Emacs => menu_text.push_str(" | ESC - Exit | "),
                 KeyScheme::Vim => {
                     if interface.in_vim_insert_mode {
-                        "McFly (Ins) | ESC - Cmd  | ⏎ - Run | TAB - Edit | F2 - Delete"
+                        menu_text.push_str(" (Ins) | ESC - Cmd | ");
                     } else {
-                        "McFly (Cmd) | ESC - Exit | ⏎ - Run | TAB - Edit | F2 - Delete"
+                        menu_text.push_str(" (Cmd) | ESC - Exit | ");
                     }
                 }
             },
-            MenuMode::ConfirmDelete => "Delete selected command from the history? (Y/N)",
+            MenuMode::ConfirmDelete => {
+                return String::from("Delete selected command from the history? (Y/N)")
+            }
         }
+
+        menu_text.push_str("⏎ - Run | TAB - Edit | ");
+
+        match interface.result_sort {
+            ResultSort::Rank => menu_text.push_str("F1 - Switch Sort to Time | "),
+            ResultSort::LastRun => menu_text.push_str("F1 - Switch Sort to Rank | "),
+        }
+
+        menu_text.push_str("F2 - Delete");
+        menu_text
     }
 
     fn bg(&self) -> String {
@@ -91,6 +105,7 @@ impl<'a> Interface<'a> {
             delete_requests: Vec::new(),
             menu_mode: MenuMode::Normal,
             in_vim_insert_mode: true,
+            result_sort: settings.result_sort.to_owned(),
         }
     }
 
@@ -133,21 +148,23 @@ impl<'a> Interface<'a> {
     }
 
     fn menubar<W: Write>(&self, screen: &mut W) {
-        let (width, _height): (u16, u16) = terminal_size().unwrap();
-        write!(
-            screen,
-            "{hide}{cursor}{clear}{fg}{bg}{text:width$}{reset_bg}",
-            hide = cursor::Hide,
-            fg = color::Fg(color::LightWhite),
-            bg = self.menu_mode.bg(),
-            cursor = cursor::Goto(1, self.info_line_index()),
-            clear = clear::CurrentLine,
-            text = self.menu_mode.text(self),
-            reset_bg = color::Bg(color::Reset),
-            width = width as usize
-        )
-        .unwrap();
-        screen.flush().unwrap();
+        if !self.settings.disable_menu {
+            let (width, _height): (u16, u16) = terminal_size().unwrap();
+            write!(
+                screen,
+                "{hide}{cursor}{clear}{fg}{bg}{text:width$}{reset_bg}",
+                hide = cursor::Hide,
+                fg = color::Fg(color::LightWhite),
+                bg = self.menu_mode.bg(),
+                cursor = cursor::Goto(1, self.info_line_index()),
+                clear = clear::CurrentLine,
+                text = self.menu_mode.text(self),
+                reset_bg = color::Bg(color::Reset),
+                width = width as usize
+            )
+            .unwrap();
+            screen.flush().unwrap();
+        }
     }
 
     fn prompt<W: Write>(&self, screen: &mut W) {
@@ -379,10 +396,17 @@ impl<'a> Interface<'a> {
             &self.input.command,
             self.settings.results as i16,
             self.settings.fuzzy,
-            &self.settings.result_sort,
+            &self.result_sort,
         );
     }
 
+    fn switch_result_sort(&mut self) {
+        match self.result_sort {
+            ResultSort::Rank => self.result_sort = ResultSort::LastRun,
+            ResultSort::LastRun => self.result_sort = ResultSort::Rank,
+        }
+    }
+
     fn select(&mut self) {
         let stdin = stdin();
         let mut screen = AlternateScreen::from(stdout().into_raw_mode().unwrap());
@@ -495,6 +519,10 @@ impl<'a> Interface<'a> {
                 self.input.insert(c);
                 self.refresh_matches();
             }
+            Key::F(1) => {
+                self.switch_result_sort();
+                self.refresh_matches();
+            }
             Key::F(2) => {
                 if !self.matches.is_empty() {
                     if self.settings.delete_without_confirm {
@@ -551,6 +579,10 @@ impl<'a> Interface<'a> {
                     self.input.insert(c);
                     self.refresh_matches();
                 }
+                Key::F(1) => {
+                    self.switch_result_sort();
+                    self.refresh_matches();
+                }
                 Key::F(2) => {
                     if !self.matches.is_empty() {
                         if self.settings.delete_without_confirm {
@@ -593,7 +625,19 @@ impl<'a> Interface<'a> {
                 Key::Char('w') => self.input.move_cursor(Move::ForwardWord),
                 Key::Char('0') | Key::Char('^') => self.input.move_cursor(Move::BOL),
                 Key::Char('$') => self.input.move_cursor(Move::EOL),
-                Key::Char('i') | Key::Char('a') => self.in_vim_insert_mode = true,
+                Key::Char('i') => self.in_vim_insert_mode = true,
+                Key::Char('I') => {
+                    self.input.move_cursor(Move::BOL);
+                    self.in_vim_insert_mode = true;
+                }
+                Key::Char('a') => {
+                    self.input.move_cursor(Move::Forward);
+                    self.in_vim_insert_mode = true;
+                }
+                Key::Char('A') => {
+                    self.input.move_cursor(Move::EOL);
+                    self.in_vim_insert_mode = true;
+                }
                 Key::Backspace => {
                     self.input.delete(Move::Backward);
                     self.refresh_matches();
@@ -607,6 +651,10 @@ impl<'a> Interface<'a> {
                 Key::Char(_c) => {
 
                 }
+                Key::F(1) => {
+                    self.switch_result_sort();
+                    self.refresh_matches();
+                },
                 Key::F(2) => {
                     if !self.matches.is_empty() {
                         if self.settings.delete_without_confirm {
