@@ -2,7 +2,7 @@ use crate::shell_history;
 use clap::AppSettings;
 use clap::{crate_authors, crate_version, value_t};
 use clap::{App, Arg, SubCommand};
-use dirs::home_dir;
+use directories_next::{ProjectDirs, UserDirs};
 use std::env;
 use std::path::PathBuf;
 use std::str::FromStr;
@@ -37,7 +37,7 @@ pub enum InterfaceView {
     Bottom,
 }
 
-#[derive(Debug)]
+#[derive(Debug, Clone)]
 pub enum ResultSort {
     Rank,
     LastRun,
@@ -86,6 +86,7 @@ pub struct Settings {
     pub delete_without_confirm: bool,
     pub interface_view: InterfaceView,
     pub result_sort: ResultSort,
+    pub disable_menu: bool,
 }
 
 impl Default for Settings {
@@ -114,6 +115,7 @@ impl Default for Settings {
             delete_without_confirm: false,
             interface_view: InterfaceView::Top,
             result_sort: ResultSort::Rank,
+            disable_menu: false,
         }
     }
 }
@@ -364,7 +366,7 @@ impl Settings {
                         &settings.mcfly_history,
                         settings.history_format,
                     )
-                    .unwrap_or_else(String::new);
+                    .unwrap_or_default();
                 }
 
                 // CD shows PWD as the resulting directory, but we want it from the source directory.
@@ -425,7 +427,7 @@ impl Settings {
                         &settings.mcfly_history,
                         settings.history_format,
                     )
-                    .unwrap_or_else(String::new)
+                    .unwrap_or_default()
                     .trim_start_matches("#mcfly: ")
                     .trim_start_matches("#mcfly:")
                     .to_string();
@@ -480,6 +482,12 @@ impl Settings {
             Some(_val) => true,
             None => false,
         };
+
+        settings.disable_menu = match env::var_os("MCFLY_DISABLE_MENU") {
+            Some(_val) => true,
+            None => false,
+        };
+
         settings.key_scheme = match env::var("MCFLY_KEY_SCHEME").as_ref().map(String::as_ref) {
             Ok("vim") => KeyScheme::Vim,
             _ => KeyScheme::Emacs,
@@ -488,17 +496,38 @@ impl Settings {
         settings
     }
 
+    // Use ~/.mcfly only if it already exists, otherwise create 'mcfly' folder in XDG_CACHE_DIR
     pub fn mcfly_training_cache_path() -> PathBuf {
-        Settings::storage_dir_path().join(PathBuf::from("training-cache.v1.csv"))
-    }
+        let cache_dir = Settings::mcfly_xdg_dir().cache_dir().to_path_buf();
 
-    pub fn storage_dir_path() -> PathBuf {
-        home_dir()
-            .unwrap_or_else(|| panic!("McFly error: Unable to access home directory"))
-            .join(PathBuf::from(".mcfly"))
+        Settings::mcfly_base_path(cache_dir).join(PathBuf::from("training-cache.v1.csv"))
     }
 
+    // Use ~/.mcfly only if it already exists, otherwise create 'mcfly' folder in XDG_DATA_DIR
     pub fn mcfly_db_path() -> PathBuf {
-        Settings::storage_dir_path().join(PathBuf::from("history.db"))
+        let data_dir = Settings::mcfly_xdg_dir().data_dir().to_path_buf();
+
+        Settings::mcfly_base_path(data_dir).join(PathBuf::from("history.db"))
+    }
+
+    fn mcfly_xdg_dir() -> ProjectDirs {
+        ProjectDirs::from("", "", "McFly").unwrap()
+    }
+
+    fn mcfly_base_path(base_dir: PathBuf) -> PathBuf {
+        Settings::mcfly_dir_in_home().unwrap_or(base_dir)
+    }
+
+    fn mcfly_dir_in_home() -> Option<PathBuf> {
+        let user_dirs_file = UserDirs::new()
+            .unwrap()
+            .home_dir()
+            .join(PathBuf::from(".mcfly"));
+
+        if user_dirs_file.exists() {
+            Some(user_dirs_file)
+        } else {
+            None
+        }
     }
 }
